local AddonName, GGL = ...
--[[-----------------------------------------------------------------------------
Label Widget
Displays text and optionally an icon.
-------------------------------------------------------------------------------]]
local Type, Version = "GoldGoblinLootGUIWidget-MBChar", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, select, pairs = math.max, select, pairs
local tonumber = tonumber
local abs, floor = math.abs, math.floor
local format = string.format

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent
local GetItemIcon, GetItemInfo = GetItemIcon, GetItemInfo

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]
local GAP_SIZE = 2
local FIX_HEIGHT = 14

local GetMoneyText = GGL.GetMoneyText

local function Frame_OnEnter(self)
    --[[
    if not self.obj.itemID then return end
    local tooltip = _G.GameTooltip
    tooltip:ClearLines()
    GGL.GUI:SetToolTipPoint(tooltip, self.obj.ttParent, self.obj.ttPoint, 2, 0)
    tooltip:SetItemByID(self.obj.itemID)
    tooltip:Show()
    ]]--
end

local function Frame_OnLeave(self)
    --_G.GameTooltip:Hide()
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
    ["OnAcquire"] = function(self)
        self:SetMoney(0, false, true)
        self.player = nil
        self:ResetToolTipPoint()
        self.imageL:Hide()
    end,

    ["OnRelease"] = function(self)

    end,

    ["OnWidthSet"] = function(self, width)

    end,

    ["SetName"] = function(self, name)
        self.name:SetText(name)
    end,

    ["SetPlayer"] = function(self, player)
        if not player then return end
        self.player = player
        self:SetName(player.displayName)
    end,

    ["SetToolTipPoint"] = function(self, ttPoint, parent)
        self.ttPoint = ttPoint or "BOTTOM_X"
        self.ttParent = parent or self.frame
    end,

    ["ResetToolTipPoint"] = function(self)
        self.ttPoint = "BOTTOM_X"
        self.ttParent = self.frame
    end,

    ["SetName"] = function(self, name)
        name = name or ""
        self.text1:SetText(name)
    end,

    ["IsLeader"] = function(self, state)
        if state then
            self.imageL:Show()
        else
            self.imageL:Hide()
        end
    end,

    ["SetMoney"] = function(self, copper, add, instantUpdate)
        self.money = add and ((self.money or 0) + copper) or (copper or 0)
        if instantUpdate then
            self.text2:SetText(GetMoneyText(self.money))
        end
        self:UpdateSortValue()
    end,

    ["UpdateSortValue"] = function(self)
        self:SetSortValue(self.money)
    end,

    ["SetSortValue"] = function(self, value)
        self.sortValue = value or 0
    end,

    ["GetSortValue"] = function(self)
        return self.sortValue
    end,

    ["UpdateListRank"] = function(self, newRank, listUpdate)
        self.text2:SetText(GetMoneyText(self.money))
        if newRank % 2 == 0 then
            self.bg:Show()
        else
            self.bg:Hide()
        end
    end,
}


--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
    local widget = {}
    widget.type  = Type

    local frame = CreateFrame("Frame", nil, UIParent)
    frame:SetHeight(FIX_HEIGHT)
    frame:SetScript("OnEnter", Frame_OnEnter)
    frame:SetScript("OnLeave", Frame_OnLeave)
    frame.obj = widget
    widget.frame = frame

    local bg = frame:CreateTexture(nil, "BACKGROUND", nil, 1)
    bg:SetAllPoints(frame)
    bg:SetColorTexture(1,1,1,0.1)
    bg:Hide()
    widget.bg = bg

    local imageL = frame:CreateTexture(nil, "BACKGROUND")
    imageL:SetPoint("LEFT", frame, "LEFT", 0, 0)
    imageL:SetWidth(FIX_HEIGHT-2)
    imageL:SetHeight(FIX_HEIGHT-2)
    imageL:SetAtlas("groupfinder-icon-leader")
    widget.imageL = imageL

    local imageR = frame:CreateTexture(nil, "BACKGROUND")
    imageR:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
    imageR:Hide()
    imageR:SetWidth(FIX_HEIGHT-2)
    imageR:SetHeight(FIX_HEIGHT-2)
    widget.imageR = imageR

    local text1 = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    text1:SetPoint("LEFT", imageL, "RIGHT", GAP_SIZE, 0)
    text1:SetPoint("RIGHT", imageR, "LEFT", -1, 0)
    text1:SetJustifyH("LEFT")
    text1:SetJustifyV("MIDDLE")
    text1:SetWordWrap(false)
    text1:SetHeight(FIX_HEIGHT)
    widget.text1 = text1

    local text2 = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    text2:SetAllPoints(text1)
    text2:SetJustifyH("RIGHT")
    text2:SetJustifyV("MIDDLE")
    text2:SetWordWrap(false)
    widget.text2 = text2

    -- methodes
    for method, func in pairs(methods) do
        widget[method] = func
    end

    return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
